import os
import tempfile
import threading


import isaac.globals as glb
import isaac.sync as sync

os.environ["PYGAME_HIDE_SUPPORT_PROMPT"] = "1"
import pygame


def mute():
    """
    sets the `event_mute` event that signals the speaker thread to stop
    speaking.
    """
    sync.event_mute.set()


def say(text: str):
    """Speaks the given text in a separate thread."""

    def say_in_thread(text: str):
        sync.event_mute.clear()
        with sync.speech_lock:
            with tempfile.NamedTemporaryFile(delete=False, suffix=".wav") as temp:
                file = temp.name
            try:
                glb.speaker.text_to_wave(text, file)
                sound = pygame.mixer.Sound(file)
                sound.play()
                while pygame.mixer.get_busy():
                    if sync.event_mute.is_set():
                        sound.stop()
                        break
                    pygame.time.wait(100)
            finally:
                os.remove(file)

    threading.Thread(target=say_in_thread, args=(text,)).start()



import threading
import queue
from isaac.types import SettingsInterface, SpeakerInteface, ListenerInterface
from typing import Optional


settings: Optional[SettingsInterface] = None
speaker: Optional[SpeakerInteface] = None
listener: Optional[ListenerInterface] = None
query_queue = queue.Queue()
event_exit = threading.Event()
past_exchanges = []



from typing import Optional, Literal
from abc import ABC, abstractmethod


class SettingsInterface(ABC):
    groq_key: Optional[str]
    groq_model: str
    gemini_key: Optional[str]
    gemini_model: str
    hearing_enabled: bool
    whisper_size: str
    speech_enabled: bool
    piper_voice: str
    response_generator: Literal["gemini", "groq"]
    system_message = Optional[str]
    context_enabled: bool
    shell: str
    prompt_tokens: int
    completion_tokens: int


class SpeakerInteface(ABC):
    @abstractmethod
    def text_to_wave(self, text: str, file: str): ...


class ListenerInterface(ABC):
    @abstractmethod
    def listen(self): ...

    @abstractmethod
    def close(self): ...

    @abstractmethod
    def pause(self): ...

    @abstractmethod
    def resume(self): ...
